# coding: utf-8
"""
  0 uid=ex2,ou=web,dc=acomea,dc=it
  # structural object class: acaUser
  objectClass: acaUser
  objectClass: top
  cn: Nome
  sn: Cognome
  mail: esempio@acomea.it
  userPassword: {MD5}bmvE5J3Ud+vJjvQEbAZ7Xw==
  uid: ex2
  displayName: nick
  g5: L
  ar: C
  newsletterg5: FALSE
  accTrattamentoDati: TRUE
  cf: DNTLSN63D07Z404Z
  createDateTime: 20130319195300Z
  #description:
  #sistemLastUpdate:
  #sistemaFirstInsert:

"""
import os
import time
import hashlib
from datetime import datetime, timedelta
from base64 import encodestring as encode

import pytz
import ldap
import pumpkin

from pumpkin import fields, filters
from pumpkin.resource import LDAPResource
from pumpkin.directory import Directory
from pumpkin.base import Model

from django.contrib.auth.models import User
from django.conf import settings
from acomea.profilo.models import AcomeaProfilo

tz_current = pytz.timezone(settings.TIME_ZONE)
tz_utc = pytz.timezone('UTC')

def get_conn():

    LDAP_RES = LDAPResource()
    LDAP_RES.server = settings.LDAP_SERVER
    LDAP_RES.login = settings.LDAP_ROOT_DN
    LDAP_RES.password = settings.LDAP_ROOT_PWD
    LDAP_RES.TLS = False
    LDAP_RES.basedn = settings.LDAP_BASE
    LDAP_RES.method = pumpkin.resource.AUTH_SIMPLE

    LDAP_CONN = Directory()
    LDAP_CONN.connect(LDAP_RES)
    return LDAP_CONN


class StringField(fields.StringField):

    """Unicode string
    """
    default = None

    def validate(self, values):
        """Check if new value is unicode
        """
        if isinstance(values, unicode):
            return values
        elif isinstance(values, basestring):
            return unicode(values)
        else:
            raise ValueError("Not a unicode value: %s" % values)


def get_user_by_cn(cn, conn=None):
    if not conn:
        conn = get_conn()
    return conn.get(AcaUser, search_filter=filters.eq('cn', cn))


def get_user_by_cf(cf, conn=None):
    if not conn:
        conn = get_conn()
    return conn.get(AcaUser, search_filter=filters.eq('cf', cf))


def get_user_by_uid(uid, conn=None):
    if not conn:
        conn = get_conn()
    return conn.get(AcaUser, search_filter=filters.eq('uid', uid))


def get_user_by_otp(otp, conn=None):
    if not conn:
        conn = get_conn()
    return conn.get(AcaUser, search_filter=filters.eq('otp', otp))


def get_user_by_email(email, conn=None):
    if not conn:
        conn = get_conn()
    return conn.get(AcaUser, search_filter=filters.eq('mail', email))


def get_user_by_email_or_uid(email, uid):
    try:
        ldap_user = get_user_by_email(email)
    except Exception:
        ldap_user = get_user_by_uid(uid)
    return ldap_user


class AcaUser(Model):

    """This is my custom model
    explained in https://docs.google.com/document/d/1qeyMkEdPCkE7171rsbFQABNQcRF-BqJYWeZTWK3OLic
    """

    _object_class_ = ['acaUser', ]
    _rdn_ = 'uid'
    first_name = StringField('cn')
    last_name = StringField('sn')
    uid = fields.StringField('uid')
    mail = StringField('mail')
    password = StringField('userPassword')
    nick = StringField('displayName')
    g5 = StringField('g5')
    ar = StringField('ar')
    newsletterg5 = fields.BooleanField('newsletterg5', true='TRUE', false='FALSE')
    acc_trattamento_dati_bool = fields.BooleanField(
        'accTrattamentoDati', true='TRUE', false='FALSE')
    acc_trattamento_dati = fields.IntegerField('accTrattamentoDatiInt')

    cf = StringField('cf')
    create_date_time = fields.GeneralizedTimeField('createDateTime')
    description = StringField('description')
    # chi ha modificato il dato
    system_last_update = StringField('systemLastUpdate')
    #: ar/gr: chi ha ha inserito il dato
    system_first_insert = StringField('systemFirstInsert')
    otp = StringField('otp')
    otp_expiry = fields.GeneralizedTimeField('otpExpiry')
    #otp_expiry = fields.DatetimeField('otpExpiry')
    activity_flag = StringField('activityFlag')
    address = StringField('address')
    city = StringField('city')
    pv = StringField('pv')
    zipcode = StringField('zipcode')
    country = StringField('country')
    locality = StringField('locality')

    def validate_password(self, pwd):
        """
        controlla la validità delle credenziali
        """
        if not pwd:
            return False
        conn = ldap.initialize(self.directory._ldapconn._uri)
        try:
            conn.bind_s(self.dn, pwd.encode('utf-8'), ldap.AUTH_SIMPLE)
            return True
        except ldap.INVALID_CREDENTIALS:
            return False

    @classmethod
    def create_from_db(cls, user, conn):
        """
        Create an ldap user from a database user
        :param user: a db user
        :param conn: an ldap connection object
        """
        dn = "uid=%s,ou=web,dc=acomea,dc=it" % user.username
        conn = conn or get_conn()
        try:
            u = cls(conn, dn)
            print "User %s already present" % user
            return
        except pumpkin.exceptions.ObjectNotFound:
            pass
        ul = cls(conn)
        ul.set_parent("ou=web,dc=acomea,dc=it")
        ul.first_name = user.first_name
        ul.last_name = user.last_name
        ul.uid = user.username
        ul.nick = user.username
        ul.mail = user.email
        ul.set_password('fake!@#$')
        ul.create_date_time = user.date_joined
        ul.system_first_insert = 'ar'
        try:
            ul.ar = user.profilo.is_cliente and 'C' or 'L'
            ul.cf = user.profilo.rac.codice_fiscale
        except AcomeaProfilo.DoesNotExist:
            pass
        ul.g5 = ul.ar
        ul.system_last_update = 'ar'

        ul.save()

        # The first time django inserts a user it just inserts username and password
        # that is not enought to insert into ldap
        # The second pass we don't have the raw password anymore, so we store it
        # in _secret when we have it, i.e. in set_password method
        if hasattr(user, '_secret'):
            ul.set_password(user._secret)
            ul.save()
        return ul

    @classmethod
    def refresh_from_db(cls, user, conn):
        """
        Refresh data of an ldap user from a database user
        """
        # _original_username impostato nel pre_save
        dn = "uid=%s,ou=web,dc=acomea,dc=it" % getattr(user, '_original_username', user.username)
        ul = cls(conn or get_conn(), dn)
        ul.first_name = user.first_name
        ul.last_name = user.last_name
        ul.uid = user.username
        ul.nick = user.username
        ul.mail = user.email
        # ul.password = user.password
        # ul.create_date_time = user.date_joined
        try:
            ul.ar = user.profilo.is_cliente and 'C' or 'L'
            ul.cf = user.profilo.rac.codice_fiscale
            if ul.cf == "":
                ul.cf = None
        except AcomeaProfilo.DoesNotExist:
            pass

        #system_last_update = 'ar'

        ul.save()
        if hasattr(ul, '_secret'):
            ul.set_password(ul._secret)
            ul.save()
        return ul

    def show_values(self):
        for key, val in self.get_attributes().iteritems():
            print key, val

    @classmethod
    def refresh_or_create_from_db(cls, user, conn=None):
        """
        Refresh or create the user
        """
        conn = conn or get_conn()
        try:
            return cls.refresh_from_db(user, conn)
        except pumpkin.exceptions.ObjectNotFound:
            return cls.create_from_db(user, conn)

    @classmethod
    def refresh_from_ldap(self, conn=None, ldap_user=None, db_user=None):
        """
        Refresh data of a db user using ldap as source

        :param conn: an ldap connection
        :param ldap_user: an ldap user to refresh
        :param a db_user: a db user whose ldap omologous is to be refreshed
        :return: db_user
        """
        # Not sync-ing passwd!
        conn = conn or get_conn()
        if not ldap_user:
            ldap_user = get_user_by_uid(db_user.username)
        if not db_user:
            db_user = User.objects.get(username=ldap_user.nick)
        db_user.first_name = ldap_user.first_name
        db_user.last_name = ldap_user.last_name
        # db_user.username = ldap_user.uid
        # db_user.username = ldap_user.nick
        db_user.email = ldap_user.mail
        db_user.date_joined = ldap_user.create_date_time or datetime.now()
        try:
            if getattr(db_user, 'profilo') and (
                not db_user.profilo.is_cliente or not db_user.profilo.is_promotore or
                not db_user.profilo.is_utente_rete or not db_user.profilo.is_acomea
            ):
                db_user.profilo.rac.codice_fiscale = ldap_user.cf
                db_user.profilo.rac.indirizzo_spedizione = ldap_user.address
                db_user.profilo.rac.citta_spedizione = ldap_user.city
                db_user.profilo.rac.provincia_spedizione = ldap_user.pv
                db_user.profilo.rac.cap_spedizione = ldap_user.zipcode
                db_user.profilo.rac.stato_spedizione = ldap_user.country
        except AcomeaProfilo.DoesNotExist:
            pass
        db_user.save()
        return db_user

    def set_password(self, secret):
        self.password = make_pass(secret.encode('utf-8'))

    def set_otp(self):
        """
        Set a one time password
        """
        self.activity_flag = 'alive'
        self.refresh_otp_expiry()
        self.otp = hashlib.sha1(str(time.time())).hexdigest()[:16]

    def refresh_otp_expiry(self):
        """
        reset opt_expiry
        """

        otp_expiry = (datetime.now(tz=tz_current) +
                           timedelta(seconds=int(settings.UTENTI_OTP_AGE or 0)))
        # pumpkin complains if I use tz, it attaches UTC if no tz is passed
        # so i first translate to UTC, then strip it
        otp_expiry = otp_expiry.astimezone(tz_utc).replace(tzinfo=None)

        self.otp_expiry = otp_expiry
        self.save()

    def reset_otp(self):
        """
        Set a one time password
        """
        self.activity_flag = 'zombie'
        self.otp_expiry = None

    def __repr__(self):
        return '<AcaUser: %s>' % self.uid


def make_pass(password, algo='SSHA', raw=False):
    """
    generate a pass suitable for openldap

    :param password: the password to be encoded
    :param algo: the algotithm to be used among: MD5, SSHA
    """
    # http://www.openldap.org/faq/data/cache/347.html
    salt = os.urandom(4)
    if algo == 'MD5':
        h = hashlib.md5(password)
    else:
        h = hashlib.sha1(password)
    h.update(salt)
    if raw:
        return h
    else:
        return u"{%s}%s" % (algo, encode(h.digest() + salt))
