# coding: utf-8
"""
.. _add_g5_users_to_group:

_add_g5_users_to_group
=============================

Command per
utilizzo::

  dj _add_g5_users_to_group

Il comando permette di aggiungere ad un gruppo impostato per memorizzare gli utenti del G5 tutti i record
trovati sull'ldap che identificano un utente che ha effettuato il primo accesso da G5.
"""

from django.core.management.base import BaseCommand
from django.conf import settings

from acomea.ldap.ldap_models import get_conn, get_user_by_email, get_user_by_uid
from acomea.contact.models import Contact, Group


class Command(BaseCommand):
    help = "Salva nel gruppo impostato per utenti G5 tutti quelli trivati in LDAP"
    output_transaction = True

    def handle(self, **options):
        try:
            conn = get_conn()
            group = Group.objects.get(id=settings.ACOMEA_LDAP_GROUP_FROM_G5)
            users = [contact.user for contact in Contact.objects.all() if contact.user]
            for i, user in enumerate(users):
                print "%s/%s" % (i, len(users))
                try:
                    username = user.username.lower()
                    if '@' in username:
                        ldap_user = get_user_by_email(username, conn=conn)
                    else:
                        ldap_user = get_user_by_uid(username, conn=conn)
                    if ldap_user and ldap_user.system_first_insert.lower() == "g5":
                        user.contact.groups.add(group)
                except Exception, e:
                    print e
        except Group.DoesNotExist:
            print "Group not found"