# coding: utf-8

import logging
import os

from django.core.management.base import BaseCommand
from codecs import open
from optparse import make_option

from acomea.ldap.ldap_models import AcaUser, get_conn
from acomea.sso.utils.sso_client import SSOClient


class Command(BaseCommand):
    """
    Sincronizza i dati ldap con l'app esterna sso/oauth SOLO UPDATE
    """
    help = "Sincronizza i dati ldap con l'app esterna sso/oauth"

    option_list = BaseCommand.option_list + (
        make_option('-b', '--begin', type='int', dest='begin', help='Begin from index'),
        make_option('-s', '--limit', type='int', dest='limit', help='Limit list retrieved'),
        make_option('-u', '--uid', dest='uid', help='Uid to manage', ),
        make_option('-f', '--file', dest='file', default=None, help="List uid from file to update on sso"),
        make_option('-o', '--output-file', dest='output-file', default=None, help="Output file name for uid in error"),
        make_option('-d', '--delete', action='store_true', default=False, help="Delete error file"),

        make_option('-c', '--console', action='store_true', default=False, help='Debug - write logging to console'),
        make_option('-l', '--debug-level', default='info', help='Set debug level (debug, info, warnings) for console'),
        make_option('-x', '--no-stop', action='store_true', default=False, help="Don't stop on error, just log"),
    )

    output_transaction = True

    logger = logging.getLogger('acomea.profilo.migrazione')

    def handle(self, **options):
        level = getattr(logging, options.get('debug_level').upper())
        self.logger.setLevel(level)

        if options.get('console'):
            console_handler = logging._handlers['console']
            console_handler.setLevel(level)
            self.logger.handlers = []
            self.logger.addHandler(console_handler)

        begin = options.get('begin')
        limit = options.get('limit')
        uid = options.get('uid')
        file_input = options.get('file')
        update_ok = 0
        update_ko = 0

        search_filter = None
        if uid:
            search_filter = '(uid={})'.format(uid)

        if file_input:
            with open(file_input, "rb") as input:
                # costruisco lista (uid=uid0)(uid=uid1)...
                search_filter = ['(uid={})'.format(uid.split(";")[0]) for uid in input]
                # costruisco str per filtro in or  (|(uid=uid0)(uid=uid1)...)
                search_filter = '(|{})'.format(''.join(search_filter))

        try:
            conn = get_conn()
            sso_client = SSOClient()

            self.logger.info('Start download users from LDAP')
            users = conn.search(AcaUser, search_filter=search_filter)

            index_begin = 0
            if begin:
                users = users[begin:]
                index_begin += begin
            if limit:
                users = users[:limit]

            error_file_name = options.get('output-file') or "/tmp/ldap_sso_update.txt"

            if options.get('delete') and os.path.exists(error_file_name):
                os.remove(error_file_name)

            with open(error_file_name, "a", 'utf-8') as error_file:
                count = len(users)
                for index, user in enumerate(users):
                    self.logger.debug(u"Try {}...\n".format(user.uid))
                    new_payload = dict()
                    try:
                        sso_instance = sso_client.detail(user.uid)
                        if user.mail != sso_instance.get('data').get('mail'):
                            new_payload['email'] = user.mail
                        if user.first_name != sso_instance.get('data').get('cn'):
                            new_payload['first_name'] = user.first_name
                        if user.last_name != sso_instance.get('data').get('sn'):
                            new_payload['last_name'] = user.last_name
                        if user.acc_trattamento_dati != sso_instance.get('data').get('accTrattamentoDatiInt'):
                            new_payload['acc_trattamento_dati_int'] = user.acc_trattamento_dati
                        if user.cf != sso_instance.get('data').get('cf'):
                            new_payload['codice_fiscale'] = user.cf
                        if user.g5 != sso_instance.get('data').get('g5'):
                            new_payload['g5'] = user.g5
                        if user.ar != sso_instance.get('data').get('ar'):
                            new_payload['ar'] = user.ar

                        data = {'message': 'No updated necessary'}
                        if new_payload:
                            data = sso_client.update(
                                username=user.uid,
                                **new_payload
                            )

                        update_ok += 1
                        self.logger.info(u"{} - {} - {} --> OK FROM SSO: {} User: {}\n".format(
                            index, (index + index_begin), count, user.uid, data['message'])
                        )

                    except Exception as e:
                        update_ko += 1
                        self.logger.error(u"ERROR FROM SSO: User: {}\t{}\t{}\n".format(user.uid, e, new_payload))
                        msg = u"{};{};{}\n".format(user.uid, e, new_payload)
                        error_file.write(msg)
                        if options.get('no_stop'):
                            pass
                        else:
                            raise

                    count -= 1
        except Exception as e:
            self.logger.fatal(e)
            raise
        self.logger.info(u"Inserted: {}\t Error: {}".format(update_ok, update_ko))

# NO BLOCK
# dj sync_from_ldap_to_sso_update_fields -ldebug -c -d -o/tmp/e.txt -x
# NO BLOCK WITH INPUT FILE
# dj sync_from_ldap_to_sso_update_fields -ldebug -c -d -f/tmp/e.txt -o/tmp/e.txt -x
# NO BLOCK WITH SIZE LIMIT
# dj sync_from_ldap_to_sso_update_fields -ldebug -c -d -o/tmp/e.txt -x -s4
