# coding: utf-8
"""
.. _sync_ldap:

sync_ldap
=============================

Command per 
utilizzo::

  dj sync_ldap

il comando permette di sincronizzare gli utenti sul db ldap e quelli 
sul db postgres nel due versi:

"-L": sincronizza gli utenti ldap su postgres
"-D": sicronizza gli utenti del db postgres in ldap

È possibile fare agire il comando su un singolo utente (opzione -i)

"""


import re
import os
import sys
import email
import tempfile
import datetime

from django.core.mail.message import EmailMessage
from django.core.management.base import BaseCommand
from django.conf import settings
from django.core import mail
from django.contrib.auth.models import User

from optparse import make_option

from acomea.utenti.ldap_models import AcaUser, get_conn

class Command(BaseCommand):
    """Get all messages from configured Host
    """
    help = """Sincronizza i dati ldap con quelli del db. Accetta opzioni per decidere la 
    direzione della sincronizzazione"""

    option_list = BaseCommand.option_list + (
        make_option('-L', '--from-ldap', action='store_true', default=False,
                    dest='from-ldap', help='Use ldap as source, database as destination'),
        make_option('-D', '--from-database', action='store_true', default=False,
                    dest='from_database', help='Use database as source, ldap as detination'),
        make_option('-l', '--limit', dest='limit', help='Limit Messages', default=1000),
        make_option('-i', '--id', dest='pk', help='Id to manage', ),
        # make_option('-s', '--skip-existent', action='store_true', default=False,
        #             help="Skip a user if it's already present. Used with --create"),
        make_option('-c', '--create', action='store_true', default=False,
                    help="Create users, don't update"),
        make_option('-x', '--no-stop', action='store_true', default=False,
                    help="Don't stop on error, just log"),
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
    )

    output_transaction = True

    def handle(self, **options):

        if not (options.get('from_database') or options.get('from_ldap')):
            print "Please specify eather --from-database or --from-ldap"
            sys.exit(1)
        source = 'ldap' if options.get('from-ldap') else 'db'
        limit = options.get('limit') or ''
        filename = options.get('file')
        pk = options.get('pk')
        qs = User.objects.all()
        if pk:
            qs = qs.filter(id=pk)
        conn = get_conn()
        for u in qs[:limit]:
            if source == 'db':
                print "Db used as source", u
                #aca = AcaUser.create_from_db(u, conn)
                try:
                    if options.get('create'):
                        aca = AcaUser.create_from_db(u, conn)
                    else:
                        aca = AcaUser.refresh_or_create_from_db(u, conn)
                except Exception, e:
                    if options.get('no_stop'):
                        sys.stderr.write("ERROR: %s User: %s\n" % (u.id, u))
                    else:
                        raise e
            else:
                print "LDAP used as source"
                try:
                    AcaUser.refresh_from_ldap(db_user=u)
                except Exception, e:
                    if options.get('no_stop'):
                        sys.stderr.write("ERROR: %s User: %s\n" % (u.id, u))
                    else:
                        raise e
                
