# coding: utf-8
from django.contrib.auth.models import User
from django.conf import settings
from django.contrib.auth.signals import user_logged_out
from django.db.models.signals import post_delete, post_save, pre_save
from acomea.profilo.models import RichiestaAttivazioneCliente

from . import ldap_models

LDAP_USER_G5_STATUS_CLIENTE = "C"


def unset_otp(sender, user, request, **kwargs):
    if not user:
        # in caso di refresh della pagina con url di logout (logout doppio), lo user è None
        return None
    ldap_user = ldap_models.get_user_by_email_or_uid(user.email, user.username)
    if ldap_user:
        ldap_user.reset_otp()
        ldap_user.save()


def delete_ldap_user(sender, **kwargs):
    user = kwargs['instance']
    ldap_user = ldap_models.get_user_by_email_or_uid(user.email, user.username)
    if ldap_user and ldap_user.ar and not ldap_user.g5 == LDAP_USER_G5_STATUS_CLIENTE:
        ldap_user.delete()


def sync_ldap(sender, **kwargs):
    obj = kwargs['instance']
    if obj.first_name:
        # The first time django saves a suer it does not yet have first/last_name
        # that are needed for ldap
        ldap_models.AcaUser.refresh_or_create_from_db(kwargs['instance'])


def sync_ldap_from_rac(sender, **kwargs):
    rac = kwargs['instance']
    ldap_models.AcaUser.refresh_or_create_from_db(rac.profilo.user)


def save_original_username(sender, **kwargs):
    obj = kwargs['instance']
    if obj.id:
        obj._original_username = User.objects.get(id=obj.id).username
old_set_password = User.set_password


def set_password(user, raw_password):
    # http://djangosnippets.org/snippets/397/
    old_set_password(user, raw_password)
    ldap_user = ldap_models.get_user_by_email_or_uid(user.email, user.username)
    if ldap_user:
        ldap_user.set_password(raw_password)
        ldap_user.save()
    user._secret = raw_password


if settings.LDAP_SERVER:
    # Replace the method
    User.set_password = set_password
    pre_save.connect(save_original_username, User)
    post_save.connect(sync_ldap, User)
    post_save.connect(sync_ldap_from_rac, RichiestaAttivazioneCliente)
    user_logged_out.connect(unset_otp)
    post_delete.connect(delete_ldap_user, sender=User)
