# -*- coding: utf-8 -*-
import datetime
import json
import logging

from django.http import HttpResponse
from django.utils import timezone
from django.views.generic import DetailView
# from django.conf import settings

from .ldap_models import get_user_by_otp

logger = logging.getLogger('acomea')


class UserDetailView(DetailView):

    def get_object(self, otp):
        user = get_user_by_otp(otp)
        now = timezone.make_aware(datetime.datetime.now(), timezone.get_default_timezone())
        # end_time = now + datetime.timedelta(seconds=int(settings.UTENTI_OTP_AGE or 0))
        # Activity_flag is not used, activity flag is a cookie now
        if user and user.otp_expiry and now <= user.otp_expiry and user.activity_flag == 'alive':
            return user
        logger.debug('Invalid OTP = {}'.format(otp))
        if user:
            logger.debug('for user = {}, otp_expiry = {}'.format(user, user.otp_expiry))
        return None

    # for Django < 1.7 use HttpResponse and json
    def render_to_response(self, context, **response_kwargs):
        response_data = context
        return HttpResponse(json.dumps(response_data), content_type="application/json")

    def get_context_data(self):
        context = {}
        if self.object:
            context = {
                'uid': self.object.uid,
                'first_name': self.object.first_name,
                'last_name': self.object.last_name,
                'codice_fiscale': self.object.cf,
                'email': self.object.mail,
                # 'otp_expiry': self.object.otp_expiry
            }
        return context

    def get(self, request, *args, **kwargs):
        otp = kwargs.get('otp')
        self.object = self.get_object(otp)
        context = self.get_context_data()
        return self.render_to_response(context)
